--[[
	MiniPet v1.2b by Carra of Stormrage
	
	There are a number of events where the player looses his pet and we can notify him:
	-Entering the game			>OK
	-Entering an instance			>OK
	-Dieing and getting back alive		>Usually OK :p
	-Using a Taxi Service			>OK
	-Teleporting/Summoning/Hearthstoning	>OK but no summoning
	
	Based on MiniPetLeash (completely rewritten except for the events which are copied). 
]]--

--Saved Variables
--See OnEvent => "Addon_loaded"
		
--Global Values
MINIPET_DEBUG = false;		--true = Debugging Mode
MiniPet_UpdateInterval = 0.2; 	--Time between two flashes of the button in seconds
MiniPet_TotalFlashTime = 10;	--Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet_PlayersPets = {};		--List of the players pet
MiniPet_CurrentPet = nil;		--Pet on the Button	
MiniPet_BoardedTaxi                    = false;
MiniPet_StartedHearthstone             = false;
MiniPet_HearthstoneComplete            = false;
MiniPet_PlayerDied                     = false;
MiniPet_PlayerReborn                   = false;
MiniPet_PlayerEnteringWorld            = true;
MiniPet_ReadyToInitialize              = false;

MiniPet_ShowButtonImg = false;		--Goes on and off if button flashes
MiniPet_FlashTime = 0;			--Time since button flashes
MiniPet_FlashButton = false;		--On when the button flashes

function MiniPet_OnLoad() 
	--Register the Events
	this:RegisterEvent("BAG_UPDATE");	
	this:RegisterEvent("ADDON_LOADED");
	
    this:RegisterEvent("PLAYER_ENTERING_WORLD");   
    this:RegisterEvent("INSTANCE_BOOT_STOP");
    this:RegisterEvent("TAXIMAP_OPENED");
    this:RegisterEvent("TAXIMAP_CLOSED");
    this:RegisterEvent("PLAYER_CONTROL_LOST");
    this:RegisterEvent("PLAYER_CONTROL_GAINED");
    this:RegisterEvent("CONFIRM_SUMMON");
    this:RegisterEvent("PLAYER_DEAD");
    this:RegisterEvent("PLAYER_ALIVE");
    this:RegisterEvent("UNIT_SPELLCAST_START");
    this:RegisterEvent("UNIT_SPELLCAST_FAILED");
    this:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
    this:RegisterEvent("UNIT_SPELLCAST_STOP");	
    this:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
    this:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
    this:RegisterEvent("UNIT_SPELLCAST_SENT");

	--Register Chat Commands
	SlashCmdList["MiniPet"] = MiniPet_ChatCommandHandler;
	SLASH_MiniPet1 = "/MiniPet";
	
	--Create the Pet Inventory
	MiniPet_GetPlayersPets();
	
	--Set a Pet
	MiniPet_ChooseRandomPet();
end--MiniPet_OnLoad();

--Set the events where you want to give the player a warning.
function MiniPet_OnEvent(event)
    if (MINIPET_DEBUG == true) then
    	if (event == "BAG_UPDATE") then
			MiniPet_dprint("MiniPet: Event seen BAG_UPDATE");	
        elseif (event == "TAXIMAP_OPENED") then
            MiniPet_dprint("MiniPet: Event seen TAXIMAP_OPENED");
        elseif (event == "TAXIMAP_CLOSED") then
            MiniPet_dprint("MiniPet: Event seen TAXIMAP_CLOSED");
        elseif (event == "PLAYER_CONTROL_LOST") then
            MiniPet_dprint("MiniPet: Event seen PLAYER_CONTROL_LOST");
        elseif (event == "PLAYER_CONTROL_GAINED") then
            MiniPet_dprint("MiniPet: Event seen PLAYER_CONTROL_GAINED");
        elseif (event == "CONFIRM_SUMMON") then
            MiniPet_dprint("MiniPet: Event seen CONFIRM_SUMMON");
        elseif (event == "PLAYER_DEAD") then
            MiniPet_dprint("MiniPet: Event seen PLAYER_DEAD");
        elseif (event == "PLAYER_ALIVE") then
            MiniPet_dprint("MiniPet: Event seen PLAYER_ALIVE");
        --elseif (event == "UNIT_SPELLCAST_START") then
            --if (arg1 == "Hearthstone") then
                --MiniPet_dprint("MiniPet: Event seen SPELLCAST_START Hearthstone");
            --end--if
            --MiniPet_dprint("MiniPet: Event seen SPELLCAST_START " .. arg1);
        --elseif (event == "UNIT_SPELLCAST_FAILED") then
            --MiniPet_dprint("MiniPet: Event seen SPELLCAST_FAILED");
        elseif (event == "UNIT_SPELLCAST_INTERRUPTED") then
            MiniPet_dprint("MiniPet: Event seen SPELLCAST_INTERRUPTED " );
        --elseif (event == "UNIT_SPELLCAST_STOP") then
            --MiniPet_dprint("MiniPet: Event seen SPELLCAST_STOP");  
        --elseif(event == "UNIT_SPELLCAST_CHANNEL_START") then
        	--MiniPet_dprint("MiniPet: Event seen UNIT_SPELLCAST_CHANNEL_START");
		elseif(event == "UNIT_SPELLCAST_SUCCEEDED") then
			MiniPet_dprint("MiniPet: Event seen UNIT_SPELLCAST_SUCCEEDED");        	
		--elseif(event == "UNIT_SPELLCAST_CHANNEL_STOP") then
			--MiniPet_dprint("MiniPet: Event seen UNIT_SPELLCAST_CHANNEL_STOP");
		elseif(event == "UNIT_SPELLCAST_SENT") then									
			MiniPet_dprint("MiniPet: Event seen UNIT_SPELLCAST_SENT " .. arg1 .. " " .. arg2);
        end--if
    end --if
    
    if (event == "ADDON_LOADED") then
		--Saved Variables
		if(MiniPet_BattleGroundCheck == nil) then
			MiniPet_BattleGroundCheck = false;	--True if you want a battleground check (no pet in bg).
			MiniPet_GiveTextWarning = true;		--True if you want a text reminder to pop a pet.
			MiniPet_GiveButtonWarning = true;	--True if you want the butotn to flash if you want to pop a pet.
			MiniPet_HideButton = false;		--True if you want the butotn hidden if you clicked it.
		end--if     
    elseif(event == "BAG_UPDATE") then
		local prePetCount = table.maxn(MiniPet_PlayersPets);
		MiniPet_GetPlayersPets();		
		local afterPetCount = table.maxn(MiniPet_PlayersPets);
		if(prePetCount ~= afterPetCount or afterPetCount == 0) then	--petCount = 0 => Set button at 0
			MiniPet_dprint("Pre & AfterCount: (" .. prePetCount .. " / " .. afterPetCount .. ")");
			MiniPet_ChooseRandomPet();
		end--if        
    elseif (event == "PLAYER_ENTERING_WORLD") then
        MiniPet_PlayerEnteringWorld = false;
        MiniPet_WarnPlayer();
    elseif (event == "TAXIMAP_CLOSED") then
        MiniPet_BoardedTaxi = true;
    elseif (event == "PLAYER_CONTROL_GAINED" and MiniPet_BoardedTaxi) then
        MiniPet_BoardedTaxi = false;
        MiniPet_WarnPlayer();
    elseif (event == "PLAYER_ALIVE" and MiniPet_HearthstoneComplete) then
        MiniPet_HearthstoneComplete = false;
        MiniPet_PlayerReborn = false;
        MiniPet_dprint("MiniPet: Activating Pet After Hearthstone!") ;
        MiniPet_WarnPlayer();    
    elseif (event == "PLAYER_DEAD") then
        MiniPet_dprint("MiniPet: MiniPet_PlayerDied") ;
        MiniPet_PlayerDied = true;
    --+TELEPORTING--        
    elseif (event == "UNIT_SPELLCAST_SENT") then
    	MiniPet_dprint("Spellcast: " .. arg2);
    	MiniPet_StartedHearthstone = false;
    	for index, spell in pairs(MINIPET_TELEPORTS) do 
    		if(arg2 == spell) then
           	 	MiniPet_dprint("MiniPet: Started Teleporting !") ;
            	MiniPet_StartedHearthstone = true;     		
    		end--if
        end--if
    elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then	
    	if(MiniPet_StartedHearthstone == true) then
	     	for index, spell in pairs(MINIPET_TELEPORTS) do 
	    		if(arg2 == spell) then
		            MiniPet_dprint("MiniPet: Finished Teleporting!");
		            MiniPet_StartedHearthstone = false;
		            MiniPet_HearthstoneComplete = true;
			    	MiniPet_WarnPlayer();		
	    		end--if
	        end--if   
        end--if
    elseif (event == "UNIT_SPELLCAST_INTERRUPTED") then
    	if(MiniPet_StartedHearthstone == true) then
	    	MiniPet_dprint("MiniPet: Interrupted Teleporting!");
    		MiniPet_StartedHearthstone = false;	   
        end--if
    end--if
    ---TELEPORTING--
    
    -- enterworld       		event = PLAYER_ENTERING_WORLD 
    -- enterinstance    		event = PLAYER_ENTERING_WORLD
    -- get off taxi     		TAXIMAP_OPENED, TAXIMAP_CLOSED, PLAYER_CONTROL_LOST, PLAYER_ALIVE, PLAYER_CONTROL_GAINED, PLAYER_ALIVE
    -- teleported       		event = PLAYER_CONTROL_LOST/PLAYER_CONTROL_GAINED  ??
    -- summoned         		event = CONFIRM_SUMMON
    -- hearthstoned/teleport  	 	UNIT_SPELLCAST_SENT (hearthstone),UNIT_SPELLCAST_SUCCEEDED(hearthstone)
    -- dies             		PLAYER_DEAD, lots of PLAYER_ALIVE (as soon as become a ghost), use UnitIsDeadOrGhost("unit") 
    -- rez              		event = PLAYER_DEAD, event = PLAYER_ALIVE, UnitIsDeadOrGhost("unit")
end--MiniPet_OnEvent(event);

function MiniPet_OnUpdate(elapsed)
    --if (MiniPet_CONFIG[ThisCharacterConfig].ENABLED) then
    if (UnitName("player") ~= "Unknown Being" and UnitName("player") ~= "Unknown Entity" and MiniPet_ReadyToInitialize) then
        MiniPet_dprint("MiniPet: MiniPet_PlayerEnteringWorld and ALIVE");
        MiniPet_PlayerEnteringWorld = false;
        MiniPet_ReadyToInitialize = false;
        MiniPet_WarnPlayer();     
    end
    if (MiniPet_PlayerDied) then
        if (not UnitIsDeadOrGhost("player")) then
            MiniPet_dprint("MiniPet: no longer deadorghost!");
            MiniPet_PlayerDied = false;
            MiniPet_WarnPlayer();
        end--if
    end--if
    
    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if(MiniPet_FlashButton == true) then
    	if(this.TimeSinceLastUpdate == nil) then 
			this.TimeSinceLastUpdate = 0;
			MiniPet_FlashTime = 0;
    	end--if
  		this.TimeSinceLastUpdate = this.TimeSinceLastUpdate + elapsed; 	
  		MiniPet_FlashTime = MiniPet_FlashTime + elapsed;
  		if(MiniPet_FlashTime < MiniPet_TotalFlashTime) then
	  		if (this.TimeSinceLastUpdate > MiniPet_UpdateInterval) then  	
	  			if(MiniPet_ShowButtonImg == true and MiniPet_CurrentPet ~= nil) then
	    			local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iItemEquipLoc, sItemTexture = GetItemInfo(MiniPet_CurrentPet);  	  			
			   		MiniPet_SetButtonImage(sItemTexture);
			   		MiniPet_ShowButtonImg = false;
			   	else
			   		MiniPet_SetButtonImage("Interface\\AddOns\\MiniPet\\Img\\MiniPet4");
			   		MiniPet_ShowButtonImg = true;
				end--if
				this.TimeSinceLastUpdate = 0;
	  		--else
	  			--MiniPet_SetButton(MiniPet_CurrentPet);
	 	 		--MiniPet_FlashButton = false;
	  		end--if
  		else
  			MiniPet_FlashButton = false;
  			MiniPet_SetButton(MiniPet_CurrentPet);
  			MiniPet_FlashTime = 0;
  		end--if
  	end--if
end--MiniPet_OnUpdate()

--Split a string, gotten from wowwiki
function MiniPet_GetCmd(msg)
	if msg then
		local a,b,c=strfind(msg, "(%S+)"); --contiguous string of non-space characters
		if a then
			return c, strsub(msg, b+2);
		else	
			return "";
		end
	end
end

--Catch the chat commands
function MiniPet_ChatCommandHandler(msg)
	if ( ( not msg ) or ( strlen(msg) <= 0) ) then		
		cprint(MINIPET_USAGE);
	else
		local command, subcommand = MiniPet_GetCmd(msg);
		if(command == "list") then
			MiniPet_ListPets();
		elseif(command == "relist") then
			MiniPet_GetPlayersPets();
		elseif(command == "toggleBg") then
			if(MiniPet_BattleGroundCheck == true) then
				MiniPet_BattleGroundCheck = false;
				cprint(MINIPET_BGENABLED);
			else
				MiniPet_BattleGroundCheck = true;
				cprint(MINIPET_BGENABLED);			
			end--if
		elseif(command == "toggleTextWarning") then
			if(MiniPet_GiveTextWarning == true) then
				MiniPet_GiveTextWarning = false;
				cprint(MINIPET_TEXTWARNINGDISABLED);
			else
				MiniPet_GiveTextWarning = true;
				cprint(MINIPET_TEXTWARNINGENABLED);
			end--if
		elseif(command == "toggleButtonWarning") then
			if(MiniPet_GiveButtonWarning == true) then
				MiniPet_GiveButtonWarning = false;
				cprint(MINIPET_BUTTONWARNINGDISABLED);
			else
				MiniPet_GiveButtonWarning = true;
				cprint(MINIPET_BUTTONWARNINGENABLED);
			end--if
		elseif(command == "toggleAutoHideButton") then
			if(MiniPet_HideButton == true) then
				MiniPet_HideButton = false;
				cprint(MINIPET_AUTOHIDEBUTTONDISABLED);
			else
				MiniPet_HideButton = true;
				cprint(MINIPET_AUTOHIDEBUTTONENABLED);
			end--if
			if(not MiniPet_Button:IsVisible()) then
				MiniPet_Button:Show();
			end--if
		elseif(command == "toggleButton") then
			if(MiniPet_Button:IsVisible()) then
				MiniPet_Button:Hide();
				cprint(MINIPET_BUTTONDISABLED);			
			else
				MiniPet_Button:Show();
				cprint(MINIPET_BUTTONENABLED);
			end--if
		elseif(command == "toggleDebug") then
			if(MINIPET_DEBUG == true) then
				MINIPET_DEBUG = false;
				cprint("Debugging off");
			else
				MINIPET_DEBUG = true;
				cprint("Debugging on");
			end
		elseif(command == "testwarning") then
			MiniPet_wprint("Test");
		else
			cprint(MINIPET_USAGE);
		end
	end
end--MiniPet_ChatCommandHandler

----------------------------
--+Pet/Inventory Functions--
--Pick a random pet and set the button.
function MiniPet_ChooseRandomPet()
	local amountOfPets = table.maxn(MiniPet_PlayersPets);
	
	if(amountOfPets > 0) then
		--math.random(upper) generates integer numbers between 1 and upper.
		local randomPetIndex = math.random(amountOfPets);
		if(amountOfPets > 1) then --Lets make sure the player gets a different pet if he gots more then one, the quick and dirty way.
			while (MiniPet_PlayersPets[randomPetIndex] == MiniPet_CurrentPet) do
				randomPetIndex = math.random(amountOfPets);	
			end--while
		end--if
		MiniPet_CurrentPet = MiniPet_PlayersPets[randomPetIndex];			
		MiniPet_SetButton(MiniPet_PlayersPets[randomPetIndex]);	
	else
		MiniPet_CurrentPet = nil;
		MiniPet_SetButton(nil);
	end
end--MiniPet_ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet_GetPlayersPets()
	MiniPet_PlayersPets = {};
	for index, pet in pairs(PETITEMNAMES) do
		local petCount = GetItemCount(pet);
		if(petCount > 0) then
			table.insert(MiniPet_PlayersPets, pet);
		end--if
	end--for
end--MiniPet_GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet_ListPets()
	cprint("MiniPet, List of your pets: ");	
	for index, pet in pairs(MiniPet_PlayersPets) do
		cprint("-|cFF00FFFF" .. pet);
	end--for
end--MiniPet_listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
--Check if a player is in a battleground
function MiniPet_isInBg()
	for index, bg in pairs(MINIPET_BATTLEGROUNDS) do
		if(GetZoneText() == bg) then
			return true;
		end--if
	end--for
	return false;
end--MiniPet_isInBg()
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet_WarnPlayer()
	if(not (MiniPet_isInBg() and MiniPet_BattleGroundCheck == true)) then--BG + option check
		if(MiniPet_GiveButtonWarning == true) then
			MiniPet_FlashButton = true;
		end--if
		if(MiniPet_GiveTextWarning == true) then	
			if(table.maxn(MiniPet_PlayersPets) > 0 ) then	--No use if player isn't cool and gots no pets!
				MiniPet_wprint(MINIPET_WARNPLAYER);
			end--if
		end--if
		if(MiniPet_HideButton == true and not MiniPet_Button:IsVisible()) then
			MiniPet_Button:Show();
		end--if		
	end--if

end--MiniPet_WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet_wprint(msg)
	MinipetTextFrameText:SetTextColor(1.0, 0.1, 0.1);
    	MinipetTextFrameText:SetText(MINIPET_WARNPLAYER);
    	MinipetTextFrame.startTime = GetTime();
    	MinipetTextFrame:Show();
end--MiniPet_wprint(msg)

--Print something to the user
function cprint(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end--cprint

--Debug Print in red
function MiniPet_dprint(msg)
	if(MINIPET_DEBUG == true) then
		DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 0.0, 0.0);
	end--if
end--dprint
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet_SetGameTooltips()
	MiniPet_Tooltip:SetOwner(MiniPet_Button, "RIGHT");    
    MiniPet_Tooltip:SetBackdropColor(0.0,0.0,0.0, 1.0); 
    if(MiniPet_CurrentPet ~= nil) then
    	MiniPet_Tooltip:SetText(MiniPet_CurrentPet);
    else
		MiniPet_Tooltip:SetText(MINIPET_NOPETS);
    end--if
end--MiniPet_SetGameTooltips()

function MiniPet_SetButton(pet)
	if(pet ~= nil) then
	    local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, iItemEquipLoc, sItemTexture = GetItemInfo(pet);   	
		MiniPet_Button:SetAttribute("type1", "item");
		--MiniPet_Button:SetAttribute("unit", "target");
	    MiniPet_Button:SetAttribute("item1", pet);	  

    	MiniPet_Button:SetNormalTexture(sItemTexture);	   
		MiniPet_Button:SetHighlightTexture(sItemTexture);    	  
    	
    	MiniPet_dprint("Pet set: " .. pet);    	
    else
		MiniPet_Button:SetAttribute("type1", "item");
		--MiniPet_Button:SetAttribute("unit", "target");
	    MiniPet_Button:SetAttribute("item1", nil);

    	MiniPet_Button:SetNormalTexture("Interface\\AddOns\\MiniPet\\Img\\MiniPet2");	
		MiniPet_Button:SetHighlightTexture("Interface\\AddOns\\MiniPet\\Img\\MiniPet2");             
    	
		MiniPet_dprint("No pets :(");
    end--if
end--MiniPet_SetButton()

function MiniPet_SetButtonImage(img)
	MiniPet_Button:SetNormalTexture(img);
	MiniPet_Button:SetHighlightTexture(img);
end--MiniPet_SetButtonImage(img)
---UI Functions--
-----------------